#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 1A.soft - Notions de SQL - correction
# 
# Correction des exercices du premier notebooks relié au SQL.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')


# ## Recupérer les données

# In[3]:


import os
if not os.path.exists("td8_velib.db3"):
    from pyensae.datasource import download_data
    download_data("td8_velib.zip", website = 'xd')
    from pyensae.sql import import_flatfile_into_database
    dbf = "td8_velib.db3"
    import_flatfile_into_database(dbf, "td8_velib.txt")                   # 2 secondes 
    import_flatfile_into_database(dbf, "stations.txt", table="stations")  # 2 minutes


# In[4]:


get_ipython().run_line_magic('load_ext', 'pyensae')


# In[5]:


get_ipython().run_line_magic('SQL_connect', 'td8_velib.db3')


# ## Exercice 1

# In[6]:


get_ipython().run_cell_magic('SQL', '', 'SELECT COUNT(*) FROM (\n    SELECT DISTINCT last_update FROM td8_velib\n) ;\n')


# In[7]:


get_ipython().run_cell_magic('SQL', '', 'SELECT MIN(last_update), MAX(last_update) FROM td8_velib ;\n')


# ## Exercice 2

# In[8]:


get_ipython().run_cell_magic('SQL', '', "SELECT number, COUNT(*) AS nb \nFROM td8_velib\nWHERE available_bikes==0 AND last_update >= '2013-09-10 11:30:19'\nGROUP BY number\nORDER BY nb DESC\n")


# ## Exercice 3 : plage horaires de cinq minutes où il n'y a aucun vélo disponible

# In[9]:


get_ipython().run_cell_magic('SQL', '', "SELECT nb, COUNT(*) AS nb_station\nFROM (\n  -- requête de l'exercice précédent\n  SELECT number, COUNT(*) AS nb \n  FROM td8_velib\n  WHERE available_bikes==0 AND last_update >= '2013-09-10 11:30:19'\n  GROUP BY number\n)\nGROUP BY nb\n")


# ## Exercice 4 : distribution horaire par station et par tranche de 5 minutes

# In[10]:


get_ipython().run_cell_magic('SQL', '', "SELECT A.number, A.heure, A.minute, 1.0 * A.nb_velo / B.nb_velo_tot  AS distribution_temporelle\nFROM (\n  SELECT number, heure, minute, SUM(available_bikes) AS nb_velo\n  FROM td8_velib\n  WHERE last_update >= '2013-09-10 11:30:19'\n  GROUP BY heure, minute, number\n) AS A\nJOIN (\n  SELECT number, heure, minute, SUM(available_bikes) AS nb_velo_tot\n  FROM td8_velib\n  WHERE last_update >= '2013-09-10 11:30:19'\n  GROUP BY number\n) AS B\nON A.number == B.number\n--WHERE A.number in (8001, 8003, 15024, 15031)  -- pour n'afficher que quelques stations\nORDER BY A.number, A.heure, A.minute\n")


# ## Zones de travail et zones de résidence

# In[11]:


get_ipython().run_cell_magic('SQL', '--df=df', "SELECT number, SUM(distribution_temporelle) AS velo_jour\nFROM (\n  -- requête de l'exercice 4\n    SELECT A.number, A.heure, A.minute, 1.0 * A.nb_velo / B.nb_velo_tot  AS distribution_temporelle\n    FROM (\n        SELECT number, heure, minute, SUM(available_bikes) AS nb_velo\n        FROM td8_velib\n        WHERE last_update >= '2013-09-10 11:30:19'\n        GROUP BY heure, minute, number\n    ) AS A\n    JOIN (\n        SELECT number, heure, minute, SUM(available_bikes) AS nb_velo_tot\n        FROM td8_velib\n        WHERE last_update >= '2013-09-10 11:30:19'\n        GROUP BY number\n    ) AS B\n    ON A.number == B.number\n)\nWHERE heure >= 10 AND heure <= 16 \nGROUP BY number\n")


# In[12]:


df = df.sort_values("velo_jour").reset_index()
df["index"] = range(0, df.shape[0])
df.head()


# In[13]:


df.plot(x="index", y="velo_jour")


# ## JOIN avec la table stations et les stations "travail"
# 
# On trouve les arrondissements où les stations de vélib sont les plus remplies en journée au centre de Paris.

# In[14]:


get_ipython().run_cell_magic('SQL', '', "SELECT C.number, name, lat, lng, velo_jour FROM \n(\n  -- requête de la partie précédente\n    SELECT number, SUM(distribution_temporelle) AS velo_jour\n    FROM (\n        -- requête de l'exercice 4\n        SELECT A.number, A.heure, A.minute, 1.0 * A.nb_velo / B.nb_velo_tot  AS distribution_temporelle\n        FROM (\n            SELECT number, heure, minute, SUM(available_bikes) AS nb_velo\n            FROM td8_velib\n            WHERE last_update >= '2013-09-10 11:30:19'\n            GROUP BY heure, minute, number\n        ) AS A\n        JOIN (\n            SELECT number, heure, minute, SUM(available_bikes) AS nb_velo_tot\n            FROM td8_velib\n            WHERE last_update >= '2013-09-10 11:30:19'\n            GROUP BY number\n        ) AS B\n        ON A.number == B.number\n    )\n    WHERE heure >= 10 AND heure <= 16 \n    GROUP BY number\n) AS C\nINNER JOIN stations\nON C.number == stations.number \n")


# In[15]: